; Code Revision Control System
; $Id: carte.asm 1.5 1997/02/12 22:19:03 marc Exp marc $
; ******************** Carte passerelle RS232 <-> I2C *******************
; Processeur : 80C652 (Philips)
; Vitesse    : 24 MHz
; Memoire    : 8 Ko ROM, 256 octets RAM interne
; Interfaces : Serie (RS232), I2C
; -----------------------------------------------------------------------
;                             Programme
; Auteur     : Marc BOUGET
; Date       : Fevrier 1997
; ***********************************************************************
; Les principes de gestion du bus sur ce micro sont tres bien expliques
; dans les doc Philips et des articles de Radio Plan. S'y reporter pour
; plus d'infos.
;

; -----------------------------------------------------------------------
; On utilise le fichier de definition du 80C552 pour le bus I2C
; -----------------------------------------------------------------------
$NOMOD51
$INCLUDE (83c552.mcu)

; Version du logiciel
VERSION equ     010h

; -----------------------------------------------------------------------
; Definition de la vitesse du bus I2C
; -----------------------------------------------------------------------
; valeur pour un bus a 25 kHz (@24MHz)
ENS1_NOSTA_STO_NOSI_AA_CR0      equ     0d4h
ENS1_NOSTA_NOSTO_NOSI_AA_CR0    equ     0c4h
ENS1_NOSTA_NOSTO_NOSI_NOAA_CR0  equ     0c0h
ENS1_STA_NOSTO_NOSI_AA_CR0      equ     0e4h

; valeur pour un bus a 94 kHz (@24MHz)
;ENS1_NOSTA_STO_NOSI_AA_CR0      equ     054h
;ENS1_NOSTA_NOSTO_NOSI_AA_CR0    equ     044h
;ENS1_NOSTA_NOSTO_NOSI_NOAA_CR0  equ     040h
;ENS1_STA_NOSTO_NOSI_AA_CR0      equ     064h

; valeur pour un bus a fond : 400khz (@24MHz)
; Deconseille : les circuits grand public ne surpportent que le 100kHz
;ENS1_NOSTA_STO_NOSI_AA_CR0      equ     0d6h
;ENS1_NOSTA_NOSTO_NOSI_AA_CR0    equ     0c6h
;ENS1_NOSTA_NOSTO_NOSI_NOAA_CR0  equ     0c2h
;ENS1_STA_NOSTO_NOSI_AA_CR0      equ     0e6h

; -----------------------------------------------------------------------
; Organisation de la RAM interne
; (adresse 80-FF adressage indirect seulement)
; -----------------------------------------------------------------------
; E0 - FF : Pile systeme (sous utilisee)
; D0 - DF : buffer de reception I2C
; C0 - CF : buffer d'emission I2C
; 90 - BF : buffer de reception RS232
; 80 - 8F : libre
; 70 - 7F : variables internes
;      7F : adresse de la carte
;      7E : adresse de l'esclave courant
;      7D : nombre d'octet a lire/ecrire sur l'I2C
;      7C : backup nbr octet a transmettre
;      7B : status de fin de transmission
;      7A : nombre de caracteres recu sur RS232
;      79 : octet de config (bit : 0=echo local,1=maintient bus, )
;      78 : flag erreur conversion hexa
;      77 : flag demandant transmission
;      76 : flag erreur
;      75 : valeur pour NOSTA_STO_NOSI_AA
;      74 : valeur pour NOSTA_NOSTO_NOSI_AA
;      73 : valeur pour NOSTA_NOSTO_NOSI_NOAA
;      72 : valeur pour STA_NOSTO_NOSI_AA
;      71 : vitesse en cours
RX_I2C  equ     0d0h
TX_I2C  equ     0c0h
RX_RS   equ     090h
OWNSLA  equ     07fh
SLA     equ     07eh
NBBYTE  equ     07dh
BACKUP  equ     07ch
FIN_TR  equ     07bh
NBCAR   equ     07ah
CONFIG  equ     079h
ERRHEX  equ     078h
REQI2C  equ     077h
ERREUR  equ     076h
NOSTA_STO_NOSI_AA       equ     075h
NOSTA_NOSTO_NOSI_AA     equ     074h
NOSTA_NOSTO_NOSI_NOAA   equ     073h
STA_NOSTO_NOSI_AA       equ     072h
SPEED   equ     071h

; -----------------------------------------------------------------------
; Definition pour l'autovectorisation de l'IT du bus I2C (cf doc Philips)
; -----------------------------------------------------------------------
PAG1    equ     001h
HADD    equ     050h

; -----------------------------------------------------------------------
; Quelques definitions utile pour le programme
; -----------------------------------------------------------------------
ENSIO1  equ     0a0h
; selection des bancs de registre
SELRB0  equ     000h
SELRB1  equ     008h
SELRB2  equ     010h
SELRB3  equ     018h
; valeur du status de fin de tr (bon, no ack, bus lost, ...)
FIN     equ     080h
FIN1    equ     040h
FIN2    equ     020h
FIN3    equ     001h

; -----------------------------------------------------------------------
; ************************** Table de vecteurs **************************
; -----------------------------------------------------------------------
; Tous les vecteurs non utilises font un RETI simple !        
	org     RESET
	jmp     INIT

	org     EXTI0
	reti

	org     TIMER0
	reti

	org     EXTI1
	reti

	org     TIMER1
	reti

	org     SINT
	reti

	org     I2CBUS
	clr     p1.4            ; allume led I2C
	push    psw
	push    S1STA           ; astuce d'auto-vectorisation
	push    HADD
	ret

; les IT suivantes ne doivent pas exister sur le 80C652 (?!)
	
; -----------------------------------------------------------------------
; ******************** Routine d'interruption bus I2C *******************
; -----------------------------------------------------------------------
; pour explications voir doc Philips et Radio Plan ...
	org     00100h
	call    fct_00
	pop     psw
	setb    p1.4            ; eteind led I2C
	reti

	org     00108h
	call    fct_08
	pop     psw
	setb    p1.4            ; eteind led I2C
	reti

	org     00110h
	call    fct_10
	pop     psw
	setb    p1.4            ; eteind led I2C
	reti    

	org     00118h
	call    fct_18
	pop     psw
	setb    p1.4            ; eteind led I2C
	reti

	org     00120h
	call    fct_20
	pop     psw
	setb    p1.4            ; eteind led I2C
	reti

	org     00128h
	call    fct_28
	pop     psw
	setb    p1.4            ; eteind led I2C
	reti

	org     00130h
	call    fct_30
	pop     psw
	setb    p1.4            ; eteind led I2C
	reti

	org     00138h
	call    fct_38
	pop     psw
	setb    p1.4            ; eteind led I2C
	reti

	org     00140h
	call    fct_40
	pop     psw
	setb    p1.4            ; eteind led I2C
	reti

	org     00148h
	call    fct_48
	pop     psw
	setb    p1.4            ; eteind led I2C
	reti

	org     00150h
	call    fct_50
	pop     psw
	setb    p1.4            ; eteind led I2C
	reti

	org     00158h
	call    fct_58
	pop     psw
	setb    p1.4            ; eteind led I2C
	reti

	org     00160h
	call    fct_60
	pop     psw
	setb    p1.4            ; eteind led I2C
	reti    

	org     00168h
	call    fct_68
	pop     psw
	setb    p1.4            ; eteind led I2C
	reti    

	org     00170h
	call    fct_70
	pop     psw
	setb    p1.4            ; eteind led I2C
	reti    

	org     00178h
	call    fct_78
	pop     psw
	setb    p1.4            ; eteind led I2C
	reti    

	org     00180h
	call    fct_80
	pop     psw
	setb    p1.4            ; eteind led I2C
	reti    

	org     00188h
	call    fct_88
	pop     psw
	setb    p1.4            ; eteind led I2C
	reti    

	org     00190h
	call    fct_90
	pop     psw
	setb    p1.4            ; eteind led I2C
	reti    

	org     00198h
	call    fct_98
	pop     psw
	setb    p1.4            ; eteind led I2C
	reti    

	org     001a0h
	call    fct_a0
	pop     psw
	setb    p1.4            ; eteind led I2C
	reti    

	org     001a8h
	call    fct_a8
	pop     psw
	setb    p1.4            ; eteind led I2C
	reti    

	org     001b0h
	call    fct_b0
	pop     psw
	setb    p1.4            ; eteind led I2C
	reti    

	org     001b8h
	call    fct_b8
	pop     psw
	setb    p1.4            ; eteind led I2C
	reti    

	org     001c0h
	call    fct_c0
	pop     psw
	setb    p1.4            ; eteind led I2C
	reti    

	org     001c8h
	call    fct_c8
	pop     psw
	setb    p1.4            ; eteind led I2C
	reti    

; Les status entre D0 et F0 n'existent pas. Le F8 vaut dire que le
; bus n'est pas active.
	org     001f8h
	call    fct_err
	pop     psw
	setb    p1.4            ; eteind led I2C
	reti    

; -----------------------------------------------------------------------
; ************************* Programme principal *************************
; -----------------------------------------------------------------------
	org     0200h
INIT:   mov     p1,#0
	mov     OWNSLA,#030h    ; initialise propre adresse
	mov     S1ADR,OWNSLA
	mov     NOSTA_STO_NOSI_AA,#054h         ; init vitesse bus 94kHz
	mov     NOSTA_NOSTO_NOSI_AA,#044h
	mov     NOSTA_NOSTO_NOSI_NOAA,#040h
	mov     STA_NOSTO_NOSI_AA,#064h
	mov     SPEED,#1
	
	mov     CONFIG,#000h    ; init config (pas echo, non maintient, ...)
	mov     b,#00           ; b contient la config
	mov     sp,#0e0h
	setb    SDA
	setb    SCL
	mov     hadd, #pag1
	orl     IEN0,#ENSIO1    ; active bus I2C
	mov     S1CON,NOSTA_NOSTO_NOSI_AA

	mov     PCON,#080h      ; double baud rate (1xxxxxxx)
	mov     TMOD,#022h      ; 8 bits auto reload
	mov     TH1,#0f3h       ; def pour 9600 bauds a 24MHz
	mov     TL1,#0f3h       ; idem
	setb    TCON.6          ; timer 1 on
	mov     S0CON,#052h     ; serial 232 on
	
	setb    p1.4            ; eteind led I2C
	setb    p1.5            ; eteind led RS232
	mov     dptr,#CRLF
	call    print
	mov     dptr,#TXT_0
	call    print           ; texte reset carte
	mov     a,#VERSION
	call    putbyte         ; numero de version
	mov     a,OWNSLA
	call    putbyte         ; adresse de la carte
	mov     a,SLA
	call    putbyte         ; adresse esclave defaut
	mov     a,SPEED
	call    putbyte         ; vitesse bus
	mov     dptr,#CRLF
	call    print


; -----------------------------------------------------------------------
; Boucle principale du programme
ever:        
	mov     ERREUR,#0       ; efface erreur
	mov     REQI2C,#0       ; efface demande transfert
	call    getline         ; attend entree rs232
	mov     a,ERREUR        ; recupere erreur
	cjne    a,#0,ever       ; si <>0 alors erreur
	call    trline          ; traite la ligne
	mov     a,ERREUR
	cjne    a,#0,ever
	mov     a,REQI2C        ; demande transfert bus ?
	cjne    a,#1,ever       ; non alors boucle
	call    trans           ; debute la transmission
	call    boucle          ; attend fin de tr
	mov     a,ERREUR
	cjne    a,#0,ev0
	mov     dptr,#TXT_OK    ; trans ok
	call    print
	jmp     ever
ev0:    cjne    a,#8,ev1
	mov     dptr,#TXT_8     ; erreur no ack
	call    print
	jmp     ever
ev1:    cjne    a,#9,ev2
	mov     dptr,#TXT_9     ; erreur bus lost
	call    print
	jmp     ever
ev2:    mov     dptr,#TXT_E     ; erreur inconnue !
	call    print
	jmp     ever            ; for ever more ...


; -----------------------------------------------------------------------
; -----------------------------------------------------------------------
; Fonctions de gestion du bus suivant le status
; -----------------------------------------------------------------------

; erreur sur le bus -> le relache
fct_00: mov     S1CON,NOSTA_STO_NOSI_AA
	mov     FIN_TR,#FIN3
	ret

; -----------------------------------------------------------------------
; MASTER STATE SERVICE ROUTINES

; start effectue -> envoie adresse (case 08)
; repeated start envoye -> envoie adresse (case 10)
fct_08: 
fct_10: mov     S1DAT,SLA
	mov     S1CON,NOSTA_NOSTO_NOSI_AA
	mov     psw,#SELRB3
	mov     r1,#TX_I2C
	mov     r0,#RX_I2C
	mov     BACKUP,NBBYTE
	ret

; -----------------------------------------------------------------------
; MASTER TRANSMITTER STATE SERVICE ROUTINES

; adresse envoye, acquit recu -> transfer octet(s)
fct_18: mov     psw,#SELRB3
	mov     S1DAT,@r1
	mov     S1CON,NOSTA_NOSTO_NOSI_AA
	inc     r1
	ret

; adresse esclave ecriture,pas d'ack -> stop bus
fct_20: mov     S1CON,NOSTA_STO_NOSI_AA
	mov     NBBYTE,BACKUP
	mov     FIN_TR,#FIN1
	ret

; ack recu sur octet transmis -> au suivant si il y en a
fct_28: djnz    NBBYTE,NOLDA1
	mov     S1CON,NOSTA_STO_NOSI_AA
	mov     FIN_TR,#FIN
	jmp     fin_28
NOLDA1: mov     psw,#SELRB3
	mov     S1DAT,@r1
	mov     S1CON,NOSTA_NOSTO_NOSI_AA
	inc     r1
fin_28: ret

; pas ack recu sur octet transmis -> stop bus
fct_30: mov     S1CON,NOSTA_STO_NOSI_AA
	ret

; bus lost -> a plus tard !
; si STA alors tentative de reprise du bus plus tard
;fct_38: mov     S1CON,STA_NOSTO_NOSI_AA
;        mov     NBBYTE,BACKUP
;        ret
; si NOSTA tentative abonne
fct_38: mov     S1CON,NOSTA_NOSTO_NOSI_AA
	mov     FIN_TR,#FIN2
	ret

; -----------------------------------------------------------------------
; MASTER RECEIVER STATE ROUTINES

; ack sur adresse lecture -> lire octets
fct_40: mov     S1CON,NOSTA_NOSTO_NOSI_AA
	ret

; pas d'ack sur adresse en lecture -> stop bus
fct_48: mov     S1CON,NOSTA_STO_NOSI_AA
	mov     NBBYTE,BACKUP
	mov     FIN_TR,#FIN1
	ret

; ack emis sur caractere en lecture -> au suivant si il en faut (sinon pas AA) ??
fct_50: mov     psw,#SELRB3
	mov     @r0,S1DAT
	djnz    NBBYTE,NOLDA2
	mov     S1CON,NOSTA_NOSTO_NOSI_NOAA
	jmp     fin_50
NOLDA2: mov     S1CON,NOSTA_NOSTO_NOSI_AA
fin_50: inc     r0
	ret

; pas d'ack emis sur donnee lue -> stop bus (fin normale lecture bus)
fct_58: mov     psw,#SELRB3
	mov     @r0,S1DAT
	mov     FIN_TR,#FIN
	mov     S1CON,NOSTA_STO_NOSI_AA
	ret

; -----------------------------------------------------------------------
; SLAVE RECEIVER STATE SERVICE ROUTINES

; recu propre adresse+W, ack emis -> reception data + ack
fct_60: mov     S1CON,NOSTA_NOSTO_NOSI_AA
	mov     psw,#SELRB3
	mov     r0,#RX_I2C
	mov     r1,#010h        ; 16 octets max
	;mov     dptr,#TXT4
	;call    print
	ret

; bus lost dans SLA+R/W master, recu own SLA+W -> reception data
; sta pour reprendre le bus apres
fct_68: mov     S1CON,STA_NOSTO_NOSI_AA
	mov     psw,#SELRB3
	mov     r0,#RX_I2C
	mov     r1,#010h
	;mov     dptr,#TXT4
	;call    print
	ret

; general call recu -> reception data
fct_70: mov     S1CON,NOSTA_NOSTO_NOSI_AA
	mov     psw,#SELRB3
	mov     r0,#RX_I2C
	mov     r1,#010h
	;mov     dptr,#txt3
	;call    print
	ret

; bus lost sur SLA+R/W master, general call recu -> reception data
; sta pour reprendre le bus apres
fct_78: mov     S1CON,STA_NOSTO_NOSI_AA
	mov     psw,#SELRB3
	mov     r0,#RX_I2C
	mov     r1,#8
	;mov     dptr,#txt3
	;call    print
	ret

; reception de donne (slave receive)
fct_80: mov     psw,#SELRB3
	mov     @r0,S1DAT
	;call    fct_dat
	djnz    r1,NOTLDAT3
	mov     S1CON,NOSTA_NOSTO_NOSI_NOAA
	ret
NOTLDAT3: mov   S1CON,NOSTA_NOSTO_NOSI_AA
	inc     r0
	ret

;slave receive sans ACK -> data not read
fct_88: mov     S1CON,NOSTA_NOSTO_NOSI_AA
	ret

; 1 data suite a general call -> un seul ACK
fct_90: mov     psw,#SELRB3
	mov     @r0,S1DAT
	;call    fct_dat
	mov     S1CON,NOSTA_NOSTO_NOSI_NOAA
	ret

; data suite a general call -> pas d'ACK
fct_98: mov     S1CON,NOSTA_NOSTO_NOSI_AA
	;mov     a,#'!'
	;call    putchar
	ret

; stop ou repeated start -> reprise surveillance bus
fct_a0: mov     S1CON,NOSTA_NOSTO_NOSI_AA
	;mov     dptr,#txt6
	;call    print
	ret

; -----------------------------------------------------------------------
; SLAVE TRANSMITTER STATE SERVICE ROUTINES

; SLA+R recu +ACK -> transmet data
fct_a8: mov     S1DAT,TX_I2C
	mov     S1CON,NOSTA_NOSTO_NOSI_AA
	mov     psw,#SELRB3
	mov     r1,#TX_I2C
	inc     r1
	;mov     dptr,#txt5
	;call    print
	ret

; bus lost SLA master, own add recu -> transmet data
; STA mis pour reprendre bus deriere
fct_b0: mov     S1DAT,TX_I2C
	mov     S1CON,STA_NOSTO_NOSI_AA
	mov     psw,#SELRB3
	mov     r1,#TX_I2C
	inc     r1
	ret

; data transmise, ack recu -> transmet data
fct_b8: mov     psw,#SELRB3
	mov     S1DAT,@r1
	mov     S1CON,NOSTA_NOSTO_NOSI_AA
	inc     r1
	;call    fct_dat
	ret

; data transmis, pas ack -> lache bus
fct_c0: mov     S1CON,NOSTA_NOSTO_NOSI_AA
	;mov     dptr,#txt6
	;call    print
	ret

; derniere donne transmise -> lache bus
fct_c8: mov     S1CON,NOSTA_NOSTO_NOSI_AA
	;mov     dptr,#txt6
	;call    print
	ret

; affiche le contenu de S1STA pour les valeur non prevue
fct_err:
	push    S1STA
	mov     dptr,#TXT_Ep
	call    print
	pop     acc
	call    putbyte
	mov     dptr,#crlf
	call    print
	mov     FIN_TR,#FIN3            ; coupe la communication
	mov     S1CON,NOSTA_NOSTO_NOSI_AA
	ret

; affiche le contenu de S1DAT
fct_dat:
	mov     a,S1DAT
	call    putbyte
	mov     a,#' '
	call    putchar
	ret

; -----------------------------------------------------------------------
; Fonction de transmission
; On en sort avec le resultat dans FIN_TR :
;  = 080h -> OK
;  = 040h -> NO ACK
;  = 020h -> BUS LOST + abandon
;  = 010h -> BUS ERROR + abandon

trans:  mov     FIN_TR,#0
	setb    STA
	ret

boucle: mov     a,FIN_TR
	jbc     acc.7,fini      ; bit fin de tr
;        jbc     acc.6,etia     ; bit de no ack
	jbc     acc.6,trnack    ; bit de non ack
	jbc     acc.5,trbusl    ; bit de bus lost
	jbc     acc.4,trberr    ; bit de bus error
	jbc     acc.0,trberr    ; erreur inconnue
	jmp     boucle
etia:   call    trans
	jmp     boucle
fini:   mov     ERREUR,#000h
	ret
trnack: mov     ERREUR,#008h
	ret
trbusl: mov     ERREUR,#009h
	ret
trberr: mov     ERREUR,#00Ah
	ret

; -----------------------------------------------------------------------
; ************************ Routines utilitaires *************************
; -----------------------------------------------------------------------
; ATTENTION : risque de problemes mapping memoire 
; si le org suivant est decommente (donc a ne pas faire !)
;        org     00500h

; -----------------------------------------------------------------------
; Envoi le texte pointe par dptr sur le port serie
print:
	clr     a
	movc    a,@a+dptr
	jz      prex
	call    PUTCHAR
	mov     a,#1
	add     a,DPL
	mov     DPL,a
	clr     a
	addc    a,dph
	mov     dph,a
	jmp     print
prex:   ret

; -----------------------------------------------------------------------
; Envoi le caractere 'a' sur le port serie
putchar:
	clr     p1.5            ; led activite rs232
	jnb     ti,$            ; attend buffer vide
	clr     ti              ; efface flag
	mov     s0buf,a         ; envoie caractere
	setb    p1.5            ; eteind led
	ret

; -----------------------------------------------------------------------
; Attend un caractere sur le port serie
getchar:
	jnb     ri,$            ; attend reception
	clr     p1.5            ; allume led rs232
	clr     ri              ; efface flag
	mov     a,s0buf         ; lit caractere
;        cjne    a,#01bh,gcex    ; si ESC alors reset
;        ljmp    00000h
;gcex:   
	;mov     a,CONFIG
	jnb     b.0,gcsui
	acall   putchar         ; envoie echo
gcsui:  setb    p1.5            ; eteind led
	ret

; -----------------------------------------------------------------------
; Ecrit l'octet 'a' en hexa sur le port serie
putbyte:
	push    acc
	swap    a
	call    hexasc
	call    putchar
	pop     acc
	call    hexasc
	call    putchar
	ret

hexasc: anl     a,#0fh
	cjne    a,#0ah,ha1
ha1:    jc      haval9
	add     a,#7
haval9: add     a,#'0'
	ret

; -----------------------------------------------------------------------
; Converti 2 caracteres pointes par r0 en octet
getbyte:
	mov     ERRHEX,#0       ; efface flag erreur
	mov     a,@r0
	inc     r0
	call    aschex          ; nibble fort
	swap    a               ; dans poid fort de a
	mov     r6,a            ; on le sauve
	mov     a,ERRHEX        ; nibble faible
	cjne    a,#0,gberr      ; erreur conv
	mov     a,@r0
	inc     r0
	call    aschex
	add     a,r6
	mov     r6,a
	mov     a,ERRHEX
	cjne    a,#0,gberr      ; erreur conv
	mov     a,r6
	ret                     ; renvoi resultat

gberr:  ret

aschex: cjne    a,#'0',ah1
ah1:    jc      aserr           ; a < '0'
	cjne    a,#'9'+1,ah2
ah2:    jc      ah09            ; 0 <= a <= 9
	cjne    a,#'A',ah3
ah3:    jc      aserr           ; 9 < a < A
	cjne    a,#'F'+1,ah4
ah4:    jc      ahaf            ; A <= a <= F
	jmp     aserr
ahaf:   clr     c
	subb    a,#007h         ; ajuste pour lettre
ah09:   clr     c
	subb    a,#'0'          ; calcul valeur
	ret

aserr:  mov     ERRHEX,#0ffh
	ret

; -----------------------------------------------------------------------
; Lit une ligne (jusqu'a CR/LF) sur le port serie
getline:
	push    psw
	mov     psw,#SELRB2     ; 2ieme bank de registres
	mov     NBCAR,#0        ; 0 caratere recu
	mov     r0,#RX_RS       ; pointeur buffer
bcget:  call    getchar
	cjne    a,#8,yget0      ; delete ?
	cjne    r0,#RX_RS,bc1   ; oui (verifie tampon)
	jmp     yget
bc1:    dec     r0              ; efface caractere
	dec     NBCAR           ; le decompte
	jmp     bcget
yget0:  call    upper           ; passe en majuscule
	mov     @r0,a           ; sauve
	inc     NBCAR           ; +1 caractere
	inc     r0
	cjne    r0,#RX_RS+36,yget      ; trop de caracteres ?
	jmp     tropget         ; oui -> on sort
yget:   cjne    a,#00dh,yget1   ; 'ENTREE' ?
	jmp     finget          ; oui -> fini
yget1:  jmp     bcget           ; au suivant

finget: ;mov     dptr,#TXT_OK
	;call    print
	dec     NBCAR           ; on ne compte pas ENTREE
	jnb     b.0,fg1         ; echo on ?
	mov     a,#00ah         ; oui -> envoi un LF
	call    putchar
fg1:    pop     psw
	ret                     ; une ligne complete

tropget:
	mov     NBCAR,#0        ; efface la ligne
	jnb     b.0,fg2         ; echo on ?
	mov     a,#00dh
	call    putchar
	mov     a,#00ah         ; oui -> envoi un LF
	call    putchar
fg2:    mov     dptr,#TXT_F     ; affiche erreur
	call    print
	mov     ERREUR,#00fh
	pop     psw
	ret

; -----------------------------------------------------------------------
; Passe le caractere 'a' en majuscule
upper:  cjne    a,#'a'-1,upp1   ; a<'a'+1 ?
	jmp     uppfin          ; a='a'-1 -> fin
upp1:   jc      uppfin          ; a<'a'-1 -> fin
	cjne    a,#'z'+1,upp2   ; a>'z'+1
	jmp     uppfin          ; a='z'+1 -> fin
upp2:   jnc     uppfin          ; a>'z'+1 -> fin
	clr     c               ; efface carry
;        subb    a,#'a'          ; a=a-'a'
;        add     a,#'A'          ; a=a+'z'
	clr     acc.5           ; idem dessus + rapide
uppfin: ret

; -----------------------------------------------------------------------
; Traite la ligne recu
trline: mov     r0,#RX_RS
	mov     r7,NBCAR
	mov     a,@r0
	inc     r0
	cjne    a,#'%',trerr    ; 1er carac = '%', non alors erreur
	mov     a,@r0           ; lit code ligne
	inc     r0
	cjne    a,#'0',trl0
	jmp     defcfg          ; (0) config carte
trl0:   cjne    a,#'1',trl1
	jmp     defadd          ; (1) definition adresse
trl1:   cjne    a,#'2',trl2
	jmp     defsla          ; (2) adresse esclave
trl2:   cjne    a,#'3',trl3
	jmp     deftx           ; (3) ecrit buffer TX
trl3:   cjne    a,#'4',trl4
	jmp     defrx           ; (4) lire octet sur esclave
trl4:   cjne    a,#'5',trl5
	jmp     defspd          ; (5) vitesse bus I2C
trl5:        
	mov     dptr,#TXT_2
	call    print           ; ordre inconnu !
	ret

trerr:  mov     dptr,#TXT_1
	call    print
	mov     NBCAR,#0
	mov     NBBYTE,#0
	mov     ERREUR,#1
	ret

; -----------------------------------------------------------------------
; Modifie (ou lit) l'octet de config de la carte
defcfg: mov     a,NBCAR
	cjne    a,#4,dcfli      ; 4 octets pour configurer
	call    getbyte         ; lit octet adresse
	mov     b,a
	mov     a,ERRHEX
	cjne    a,#0,dcfhx      ; erreur dans conv
	mov     CONFIG,b        ; affecte config
	mov     dptr,#TXT_OK
	call    print
	ret
dcfli:  cjne    a,#2,dcferr     ; si pas 2 octets erreur
	mov     dptr,#TXT_0
	call    print
	mov     a,b
	call    putbyte
	mov     dptr,#CRLF
	call    print
	ret
dcfhx:  mov     dptr,#TXT_4
	call    print
	mov     ERREUR,#4
	ret
dcferr: mov     dptr,#TXT_3
	call    print
	mov     ERREUR,#3
	ret

; -----------------------------------------------------------------------
; Modifie l'adresse de la carte
defadd: mov     a,NBCAR
	cjne    a,#2,dad1
	jmp     dadshw          ; affiche adresse courante
dad1:   cjne    a,#4,daderr     ; 4 octets pour defadd !
	call    getbyte         ; lit octet adresse
	mov     r7,a
	mov     a,ERRHEX
	cjne    a,#0,dadhx      ; erreur dans conv
	mov     OWNSLA,r7       ; affecte nouvelle adresse
	mov     S1ADR,OWNSLA
	mov     dptr,#TXT_OK
	call    print
	ret

dadshw: mov     dptr,#TXT_0     ; si %2 sans parametre
	call    print
	mov     a,OWNSLA
	call    putbyte         ; affiche adresse courante
	mov     dptr,#CRLF
	call    print
	mov     ERREUR,#0
	ret

daderr: mov     dptr,#TXT_3
	call    print
	mov     ERREUR,#3
	ret
dadhx:  mov     dptr,#TXT_4
	call    print
	mov     ERREUR,#4
	ret

; -----------------------------------------------------------------------
; Change l'adresse esclave (+R/W bit)
defsla: mov     a,NBCAR
	cjne    a,#2,dsl1
	jmp     dslshw          ; affiche adresse esclave
dsl1:   cjne    a,#4,dslerr     ; 4 octets pour defsla !
	call    getbyte         ; lit octet adresse
	mov     r7,a
	mov     a,ERRHEX
	cjne    a,#0,dslhx      ; erreur dans conv
	mov     SLA,r7          ; affecte nouvelle adresse
	mov     dptr,#TXT_OK
	call    print
	ret

dslshw: mov     dptr,#TXT_0     ; si %2 sans parametre
	call    print
	mov     a,SLA
	call    putbyte         ; affiche adresse esclave
	mov     dptr,#CRLF
	call    print
	mov     ERREUR,#0
	ret

dslerr: mov     dptr,#TXT_3
	call    print
	mov     ERREUR,#3
	ret
dslhx:  mov     dptr,#TXT_4
	call    print
	mov     ERREUR,#4
	ret

; -----------------------------------------------------------------------
; Rempli buffer emission + envoi
deftx:  mov     REQI2C,#0
	mov     a,NBCAR
	cjne    a,#2,dtsui
	jmp     dtxerr          ; il faut au moins 1 data
dtsui:  jb      acc.0,dtxerr    ; si nombre impair de carac, pas bon
	mov     r5,NBCAR
	dec     r5
	dec     r5              ; saute 2 premier carac
	mov     r1,#TX_I2C
	mov     NBBYTE,#0
dtxbc:  call    getbyte
	mov     r6,a            ; sauve conv
	mov     a,ERRHEX
	cjne    a,#0,dtxhx      ; erreur conv
	mov     a,r6
	mov     @r1,a           ; octet dans buffer
	inc     r1
	inc     NBBYTE          ; +1 octet dans buffer
	dec     r5
	djnz    r5,dtxbc        ; au suivant
	mov     a,SLA
	clr     acc.0           ; SLA+W
	mov     SLA,a
	mov     REQI2C,#1       ; temoin demande transfert
	ret

dtxhx:  mov     dptr,#TXT_4
	call    print
	mov     ERREUR,#4
	ret

dtxerr: mov     dptr,#TXT_3        
	call    print
	mov     ERREUR,#3
	ret

; -----------------------------------------------------------------------
; Lecture de n octet sur esclave
defrx:  mov     a,NBCAR
	cjne    a,#4,dslerr     ; 4 octets pour defrx !
	call    getbyte         ; lit nbr octet
	mov     r7,a
	mov     a,ERRHEX
	cjne    a,#0,dslhx      ; erreur dans conv
	mov     a,r7
	cjne    a,#17,drx1
drx1:   jnc     drxnbr          ; pas plus de 16 octets
	cjne    a,#0,drx2
	jnc     drxnbr          ; mais au moins 1
drx2:   mov     a,SLA           ; recupere adresse courante
	setb    acc.0           ; met bit lecture
	mov     SLA,a
	mov     NBBYTE,r7       ; compteur d'octets
	call    trans           ; debute la transmission
	call    boucle          ; attend fin de tr
	mov     a,ERREUR
	cjne    a,#0,drx3
	mov     dptr,#TXT_0
	call    print
	clr     c
	mov     a,r7            ; nombre octet de depart
	subb    a,NBBYTE        ; - nbr transmis = recu
	mov     r7,a
	mov     r0,#RX_I2C      ; pointe buffer
drxbc:  mov     a,@r0
	call    putbyte         ; les affiches 1 par 1
	inc     r0
	djnz    r7,drxbc
	mov     dptr,#CRLF
	call    print
	mov     ERREUR,#0
	jmp     drxfin
drx3:   cjne    a,#8,drx4
	mov     dptr,#TXT_8     ; erreur no ack
	call    print
	jmp     drxfin
drx4:   cjne    a,#9,drx5
	mov     dptr,#TXT_9     ; erreur bus lost
	call    print
	jmp     drxfin
drx5:   mov     dptr,#TXT_E     ; erreur inconnue !
	call    print
drxfin: ret

drxnbr: mov     ERREUR,#3
	mov     dptr,#TXT_3
	call    print
	ret

; -----------------------------------------------------------------------
; Change la vitesse du bus (ou affiche)
defspd: mov     a,NBCAR
	cjne    a,#2,dsp1
	jmp     dspshw          ; affiche adresse courante
dsp1:   cjne    a,#4,dsperr     ; 4 octets pour defadd !
	call    getbyte         ; lit octet vitesse
	mov     r7,a
	mov     a,ERRHEX
	jnz     dsphx           ; erreur conv
	mov     a,r7
	cjne    a,#7,dsp2
dsp2:   jnc     dsperr          ; vitesse max = 6 (de 0 a 6)
	mov     SPEED,r7        ; sauve vitesse
	
	mov     r0,#NOSTA_STO_NOSI_AA
	mov     r6,#4
dsp3:   mov     a,@r0
	anl     a,#07ch
	mov     @r0,a
;        anl     @r0,#07Ch       ; efface vitesse
	dec     r0
	djnz    r6,dsp3

	mov     dptr,#SPREF     ; table des vitesse
	mov     r0,#NOSTA_STO_NOSI_AA
	mov     r6,#4
dsp4:   clr     a
	movc    a,@a+dptr       ; lit valeur data
	orl     a,@r0           ; affecte vitesse
	mov     @r0,a
	dec     r0
	mov     a,#1            ; table+1
	add     a,DPL
	mov     DPL,a
	clr     a
	addc    a,dph
	mov     dph,a
	djnz    r6,dsp4
	mov     dptr,#TXT_OK
	call    print
	mov     ERREUR,#0
	ret

;       mov     NOSTA_STO_NOSI_AA,
;       mov     NOSTA_NOSTO_NOSI_AA,
;       mov     NOSTA_NOSTO_NOSI_NOAA,
;       mov     STA_NOSTO_NOSI_AA,

dspshw: mov     dptr,#TXT_0
	call    print
	mov     a,SPEED
	call    putbyte
	mov     dptr,#CRLF
	call    print
	mov     ERREUR,#0
	ret

dsphx:  mov     dptr,#TXT_4
	call    print
	mov     ERREUR,#3
	ret
dsperr: mov     dptr,#TXT_3        
	call    print
	mov     ERREUR,#3
	ret
; -----------------------------------------------------------------------


; -----------------------------------------------------------------------
; Textes des codes de retour
; 0: OK                       8: No ack
; 1: Syntaxe (entete)         9: Bus lost
; 2: Ordre inconnu            A: Bus error
; 3: Taille invalide          B:
; 4: Valeur non hexa          C:
; 5:                          D:
; 6:                          E: Erreur inconuue !
; 7:                          F: Trop de caracteres
TXT_OK: db      "!0",00dh,00ah,0
TXT_1:  db      "!1",00dh,00ah,0
TXT_2:  db      "!2",00dh,00ah,0
TXT_3:  db      "!3",00dh,00ah,0
TXT_4:  db      "!4",00dh,00ah,0
TXT_5:  db      "!5",00dh,00ah,0
TXT_6:  db      "!6",00dh,00ah,0
TXT_7:  db      "!7",00dh,00ah,0
TXT_8:  db      "!8",00dh,00ah,0
TXT_9:  db      "!9",00dh,00ah,0
TXT_A:  db      "!A",00dh,00ah,0
TXT_B:  db      "!B",00dh,00ah,0
TXT_C:  db      "!C",00dh,00ah,0
TXT_D:  db      "!D",00dh,00ah,0
TXT_E:  db      "!E",00dh,00ah,0
TXT_F:  db      "!F",00dh,00ah,0
TXT_0:  db      "!0",0
TXT_Ep: db      "!E",0
CRLF:   db      00dh,00ah,0
SPREF:  db      080h,000h,001h,002h,003h,081h,082h
CREF:   db      054h,044h,040h,064h
	
	end
