/* i2c_552.c : I2C interrupt routine */
/* (c) Marc BOUGET - 1998 */

#include "reg552.h"

/* Special definitions for a 94 kHz bus (@24MHz) */
#define NOSTA_STO_NOSI_AA	0x54
#define NOSTA_NOSTO_NOSI_AA	0x44
#define NOSTA_NOSTO_NOSI_NOAA	0x40
#define STA_NOSTO_NOSI_AA	0x64


/* Special definitions for a 25 kHz bus (@24MHz) */
/*#define NOSTA_STO_NOSI_AA	0xd4
#define NOSTA_NOSTO_NOSI_AA	0xc4
#define NOSTA_NOSTO_NOSI_NOAA	0xc0
#define STA_NOSTO_NOSI_AA	0xe4
*/

char I2C_own_address;		/* C552 own address on I2C */
char I2C_slave;			/* current slave address */
char I2C_nbbyte;		/* nbr of byte to transfert */
char I2C_nbbyte_bk;		/* backup on bus error */
char I2C_status;		/* status for end of transmission */
#define I2C_OK		1
#define I2C_NOACK	2
#define I2C_LOST	3
#define I2C_BERR	4
#define I2C_RUN		9
char idata I2C_buffer[10];
char *I2C_ptr;

int cpt;
char post;

void SerialPut (unsigned char ch) reentrant;

void I2C_IT(void) interrupt 6 using 3
{

	switch(S1STA) {

	case 0 :	/* bus error ! */
		S1CON = NOSTA_STO_NOSI_AA;
		I2C_status = I2C_BERR;
		break;

	case 0x08:	/* (repeated)start done -> send address */
	case 0x10:
		S1DAT = I2C_slave;
		S1CON = NOSTA_NOSTO_NOSI_AA;
		I2C_ptr = I2C_buffer;
		I2C_nbbyte_bk = I2C_nbbyte;
		break;

/*---------- Master transmitter ----------*/
	case 0x18:	/* address send, ACK ok -> send bytes */
		S1DAT = *I2C_ptr++;
		I2C_nbbyte--;
		S1CON = NOSTA_NOSTO_NOSI_AA;
		break;

	case 0x20:	/* address send, no ACK -> stop bus */
		S1CON = NOSTA_STO_NOSI_AA;
		I2C_status = I2C_NOACK;
		break;

	case 0x28:	/* ACK ok on byte -> send next one (if any) */
		if(I2C_nbbyte--) {
			S1DAT = *I2C_ptr++;
			S1CON = NOSTA_NOSTO_NOSI_AA;
		} else {
			S1CON = NOSTA_STO_NOSI_AA;
			I2C_status = I2C_OK;
		}
		break;

	case 0x30:	/* no ACK on byte -> stop bus */
		S1CON = NOSTA_STO_NOSI_AA;
		I2C_status = I2C_NOACK;
		break;

	case 0x38:	/* bus lost -> shoot again later ! */
		S1CON = NOSTA_NOSTO_NOSI_AA;
		I2C_status = I2C_LOST;
		break;

/*---------- Master Receiver ----------*/
	case 0x40:	/* ACK on address -> read byte */
		S1CON = NOSTA_NOSTO_NOSI_AA;
		break;

	case 0x48:	/* no ACK on address -> stop bus */
		S1CON = NOSTA_STO_NOSI_AA;
		I2C_status = I2C_NOACK;
		break;

	case 0x50:	/* ACK send -> next one (if any, else not AA) */
		*I2C_ptr++ = S1DAT;
		if(I2C_nbbyte--)
			S1CON = NOSTA_NOSTO_NOSI_AA;
		else
			S1CON = NOSTA_NOSTO_NOSI_NOAA;
		break;

	case 0x58:	/* no ACK send -> stop bus (normal end) */
		*I2C_ptr++ = S1DAT;
		S1CON = NOSTA_STO_NOSI_AA;
		I2C_status = I2C_OK;
		break;

/*---------- Slave Receiver ----------*/
	case 0x60:	/* received own address+W, ACK done -> receive data with ACK */
		S1CON = NOSTA_NOSTO_NOSI_AA;
		break;

	case 0x68:	/* bus lost, received own address+W, ACK done -> receive data with ACK */
		S1CON = STA_NOSTO_NOSI_AA;
		break;

	case 0x70:	/* general call received -> receive data */
		S1CON = NOSTA_NOSTO_NOSI_AA;
		break;

	case 0x78:	/* bus lost, general call received -> receive data */
		S1CON = STA_NOSTO_NOSI_AA;
		break;

	case 0x80:	/* receive data */
		*I2C_ptr++ = S1DAT;
		if(I2C_nbbyte > 0)
			S1CON = NOSTA_NOSTO_NOSI_AA;
		else
			S1CON = NOSTA_NOSTO_NOSI_NOAA;
		break;

	case 0x88:	/* slave receive without ACK -> data not read */
		S1CON = NOSTA_NOSTO_NOSI_AA;
		break;

	case 0x90:	/* 1st data after General call -> only 1 ACK */
		*I2C_ptr++ = S1DAT;
		S1CON = NOSTA_NOSTO_NOSI_NOAA;
		break;

	case 0x98:	/* data after G.C. -> no ACK */
		S1CON = NOSTA_NOSTO_NOSI_AA;
		break;

	case 0xA0:	/* start or repeated start -> standby */
		S1CON = NOSTA_NOSTO_NOSI_AA;
		break;

/*---------- Slave Transmitter ---------*/
	case 0xA8:	/* SLA+R+ACK received -> transmit data */
		S1DAT = *I2C_ptr++;
		S1CON = NOSTA_NOSTO_NOSI_AA;
		break;

	case 0xB0:	/* bus lost, SLA received -> transmit data */
		S1DAT = *I2C_ptr++;
		S1CON = STA_NOSTO_NOSI_AA;
		break;

	case 0xB8:	/* data send, ACK OK -> next data */
		S1DAT = *I2C_ptr++;
		S1CON = NOSTA_NOSTO_NOSI_AA;
		break;

	case 0xC0:	/* data send, no ACK -> stop bus */
		S1CON = NOSTA_NOSTO_NOSI_AA;
		break;

	case 0xC8:	/* last data send -> stop bus */
		S1CON = NOSTA_NOSTO_NOSI_AA;
		break;
	}

}

unsigned char SerialGet () reentrant
{
	while (!RI)
		/* nop*/;
	RI = 0;
	return S0BUF;
}

void SerialPut (unsigned char ch) reentrant
{
	while (!TI)
		/* nop */;
	TI = 0;
	S0BUF = ch ;
}

void SerialPrint (char _code *txt)
{
	while(*txt)
		SerialPut(*txt++);
}

void PutHex(unsigned char x)
{
	unsigned char a;

	a = x>>4;
	a += 48;
	if(a > 57)
		a += 7;
	SerialPut(a);

	a = x &0x0f;
	a += 48;
	if(a > 57)
		a += 7;
	SerialPut(a);
}


/* Set start condition and wait for end of data transfert */
char I2C_trans(void)
{
	char i;

	I2C_status = I2C_RUN;
	i = I2C_status;
	STA = 1;		/* make start */
	while(i == I2C_RUN)
		i = I2C_status;
	return I2C_status;
}



void I2C_init(void)
{
	S1ADR = 0x30;			/* init own address */
	SDA = 1;
	SCL = 1;			/* set I2C lines to iddle */
	IEN0 |= 0xa0;			/* enable I2C interrupt */

	S1CON = NOSTA_NOSTO_NOSI_AA;	/* start I2C interface */
}


/*********************************************************************/

#define CPT 10000

void Timer0_IT(void) interrupt 2 using 2
{
	if( !(cpt--) ) {
		post = 1;
		cpt = CPT;
	}
}

main()
{
	int i;
	unsigned char a, b;

	CT0I = 0;

	I2C_init();

	SerialPrint("Carte 80C552 I2C - Marc BOUGET 1998");
	SerialPut(0x0d);
	SerialPut(0x0a);
	SerialPrint("Circuit PCF8574 : 0x");
	a = 0x40;
	PutHex(a);
	SerialPut(0x0d);
	SerialPut(0x0a);

	I2C_slave = 0x41;
	I2C_buffer[0] = 255;
	I2C_buffer[1] = 0;
	TH0 = 6;
	TL0 = 6;
	TMOD = 0x22;
	TR0 = 1;
	IEN0 |= 0x02;

	post = 0;
	cpt = CPT;

	//I2C_slave = 0x90;
	//I2C_nbbyte = 1;
	//I2C_buffer[0] = 0;
	//I2C_trans();

	while(1) {
		P1 = P1 ^ 0x01;         /* toggle LED on P1.0 */

		I2C_slave = 0x91;	/* read T from LM75 */
		I2C_nbbyte = 2;
		I2C_trans();
		I2C_slave = 0x40;	/* send MSB to PCF8574 */
		I2C_nbbyte = 1;
		I2C_trans();
		a = I2C_buffer[0];
		SerialPrint("Temperature : 0x");
		PutHex(a);		/* print it on serial */
		SerialPut(0x0d);
		SerialPut(0x0a);


		while(post==0)
			/* nop */;	/* wait for timer */
		post = 0;
	}
}

