/**************************************************************************/
/*                                                                        */     
/*                                                                        */        
/*       80C552 Processor Declarations                                    */
/*                                                                        */     
/*                                                                        */     
/**************************************************************************/

#ifndef _C552_
#define _C552_


/***************************  BYTE Registers ******************************/

sfr at 0x80	P0	;  /* Port 0                  */
sfr at 0x90	P1	;  /* Port 1                  */
sfr at 0xA0	P2	;  /* Port 2                  */
sfr at 0xB0	P3	;  /* Port 3                  */
sfr at 0xC0	P4	;  /* Port 4                  */
sfr at 0xC4	P5	;  /* Port 5                  */
sfr at 0xD0	PSW	;  /* Program Status Word     */
sfr at 0xE0	ACC	;  /* Accumulator             */
sfr at 0xF0	B	;  /* Multiplication Register */
sfr at 0x81	SP	;  /* Stack Pointer           */
sfr at 0x82	DPL	;  /* Data Pointer Low Byte   */
sfr at 0x83	DPH	;  /* Data Pointer Hi Byte    */
sfr at 0x87	PCON	;  /* Power Control           */
sfr at 0x88	TCON	;  /* Timer Control           */
sfr at 0x89	TMOD	;  /* Timer Mode              */
sfr at 0x8A	TL0	;  /* Timer 0 Low Byte        */
sfr at 0x8B	TL1	;  /* Timer 1 Low Byte        */
sfr at 0x8C	TH0	;  /* Timer 0 Hi Byte         */
sfr at 0x8D	TH1	;  /* Timer 1 Hi Byte         */
sfr at 0x98	S0CON	;  /* Serial Port 0 Control   */
sfr at 0x99	S0BUF	;  /* Serial Port 0 Buffer    */
sfr at 0xA8	IEN0	;  /* Int Enable Reg 0        */
sfr at 0xA9	CML0	;  /* Compare 0 Low Byte      */
sfr at 0xAA	CML1	;  /* Compare 1 Low Byte      */
sfr at 0xAB	CML2	;  /* Compare 2 Low Byte      */
sfr at 0xAC	CTL0	;  /* Capture 0 Low Byte      */
sfr at 0xAD	CTL1	;  /* Capture 1 Low Byte      */
sfr at 0xAE	CTL2	;  /* Capture 2 Low Byte      */
sfr at 0xAF	CTL3	;  /* Capture 3 Low Byte      */
sfr at 0xB8	IP0	;  /* Int Priority Reg 0      */
sfr at 0xC5	ADCON	;  /* A/D Converter Control   */
sfr at 0xC6	ADCH	;  /* A/D Converter Hi Byte   */
sfr at 0xC8	TM2IR	;  /* Timer 2 Interupt Flags  */
sfr at 0xC9	CMH0	;  /* Compare 0 Hi Byte       */
sfr at 0xCA	CMH1	;  /* Compare 1 Hi Byte       */
sfr at 0xCB	CMH2	;  /* Compare 2 Hi Byte       */
sfr at 0xCC	CTH0	;  /* Capture 0 Hi Byte       */
sfr at 0xCD	CTH1	;  /* Capture 1 Hi Byte       */
sfr at 0xCE	CTH2	;  /* Capture 2 Hi Byte       */
sfr at 0xCF	CTH3	;  /* Capture 3 Hi Byte       */
sfr at 0xD8	S1CON	;  /* Serial Port 1 Control   */
sfr at 0xD9	S1STA	;  /* Serial Port 1 Status    */
sfr at 0xDA	S1DAT	;  /* Serial Port 1 Data      */
sfr at 0xDB	S1ADR	;  /* Serial Port 1 Slave Add */
sfr at 0xE8	IEN1	;  /* Int Enable Reg 1        */
sfr at 0xF8	IP1	;  /* Int Priority Reg 1      */
sfr at 0xEA	TM2CON	;  /* Timer 2 Counter Control */
sfr at 0xEB	CTCON	;  /* Capture Control         */
sfr at 0xEC	TML2	;  /* Timer 2 Low Byte        */
sfr at 0xED	TMH2	;  /* Timer 2 Hi Byte         */
sfr at 0xEE	STE	;  /* Set Enable              */
sfr at 0xEF	RTE	;  /* Reset/Toggle Enable     */
sfr at 0xFC	PWM0	;  /* Pulse Width Register 0  */
sfr at 0xFD	PWM1	;  /* Pulse Width Register 1  */
sfr at 0xFE	PWMP	;  /* Prescaler Freq Control  */
sfr at 0xFF	T3	;  /* Watchdog Timer          */

/***************************  BIT Registers *******************************/

sbit at 0x8F	TF1	;  /* TCON.7      Timer 1 Overflow Flag     */
sbit at 0x8E	TR1	;  /* TCON.6      Timer 1 On/Off Control    */
sbit at 0x8D	TF0	;  /* TCON.5      Timer 0 Overflow Flag     */
sbit at 0x8C	TR0	;  /* TCON.4      Timer 0 On/Off Control    */
sbit at 0x8B	IE1	;  /* TCON.3      Ext Interupt 1 Edge Flag  */
sbit at 0x8A	IT1	;  /* TCON.2      Ext Interupt 1 Type       */
sbit at 0x89	IE0	;  /* TCON.1      Ext Interupt 0 Edge Flag  */
sbit at 0x88	IT0	;  /* TCON.0      Ext Interupt 0 Type       */

sbit at 0x90	CT0I	;  /* P1.0        Capture/Timer Input 0     */
sbit at 0x91	CT1I	;  /* P1.1        Capture/Timer Input 1     */
sbit at 0x92	CT2I	;  /* P1.2        Capture/Timer Input 2     */
sbit at 0x93	CT3I	;  /* P1.3        Capture/Timer Input 3     */
sbit at 0x94	T2	;  /* P1.4        T2 Event Input            */
sbit at 0x95	RT2	;  /* P1.5        T2 Timer Reset Signal     */
sbit at 0x96	SCL	;  /* P1.6        Serial Port Clock Line I2C*/
sbit at 0x97	SDA	;  /* P1.7        Serial Port Data Line I2C */

sbit at 0x98	RI	;  /* S0CON.0     Reveive Interupt Flag     */
sbit at 0x99	TI	;  /* S0CON.1     Transmit Interupt Flag    */
sbit at 0x9A	RB8	;  /* S0CON.2     Receive Bit 8             */
sbit at 0x9B	TB8	;  /* S0CON.3     Transmit Bit 8            */
sbit at 0x9C	REN	;  /* S0CON.4     Receive Enable            */
sbit at 0x9D	SM2	;  /* S0CON.5     Serial Mode Control Bit 2 */
sbit at 0x9E	SM1	;  /* S0CON.6     Serial Mode Control Bit 1 */ 
sbit at 0x9F	SM0	;  /* S0CON.7     Serial Mode Control Bit 0 */

sbit at 0xAF	EA	;  /* IEN0.7      Global Interupt Enable    */
sbit at 0xAE	EAD	;  /* IEN0.5      Enable A/D Interupt       */
sbit at 0xAE	ES1	;  /* IEN0.6      Serial Port 1 Int Enable  */
sbit at 0xAC	ES0	;  /* IEN0.4      Serial Port 0 Int Enable  */
sbit at 0xAB	ET1	;  /* IEN0.3      Timer 1 Interupt Enable   */
sbit at 0xAA	EX1	;  /* IEN0.2      Ext Interupt 1 Enable     */
sbit at 0xA9	ET0	;  /* IEN0.1      Timer 0 Interupt Enable   */
sbit at 0xA8	EX0	;  /* IEN0.0      Ext Interupt 0 Enable     */

sbit at 0xBE	PAD	;  /* IP0.6       A/D Priority              */
sbit at 0xBD	PS1	;  /* IP0.5       SIO 1 Priority            */
sbit at 0xBC	PS0	;  /* IP0.4       SIO 0 Priority            */
sbit at 0xBB	PT1	;  /* IP0.3       Timer 1 Priority          */
sbit at 0xBA	PX1	;  /* IP0.2       Ext Interupt 1 Priority   */
sbit at 0xB9	PT0	;  /* IP0.1       Timer 0 Priority          */
sbit at 0xB8	PX0	;  /* IP0.0       Ext Interupt 0 Priority   */

sbit at 0xB7	RD	;  /* P3.7        Read Enable               */
sbit at 0xB6	WR	;  /* P3.6        Write Enable              */
sbit at 0xB5	T1	;  /* P3.5        Timer 1 Count Input       */
sbit at 0xB4	T0	;  /* P3.4        Timer 0 Count Input       */
sbit at 0xB3	INT1	;  /* P3.3        Ext Interupt 1 Input      */
sbit at 0xB2	INT0	;  /* P3.2        Ext Interupt 0 Input      */
sbit at 0xB1	TXD	;  /* P3.1        Serial Port Transmit      */
sbit at 0xB0	RXD	;  /* P3.0        Serial Port Receive       */

sbit at 0xCF	T2OV	;  /* TM2IR.7     T2 Overflow               */
sbit at 0xCE	CMI2	;  /* TM2IR.6     T2 Comparator 2           */
sbit at 0xCD	CMI1	;  /* TM2IR.5     T2 Comparator 1           */
sbit at 0xCC	CMI0	;  /* TM2IR.4     T2 Comparator 0           */
sbit at 0xCB	CTI3	;  /* TM2IR.3     T2 Capture 3              */
sbit at 0xCA	CTI2	;  /* TM2IR.2     T2 Capture 2              */
sbit at 0xC9	CTI1	;  /* TM2IR.1     T2 Capture 1              */
sbit at 0xC8	CTI0	;  /* TM2IR.0     T2 Capture 0              */

sbit at 0xC7	CMT1	; /* P4.7        T2 Compare & Toggle Out   */
sbit at 0xC6	CMT0	; /* P4.6        T2 Compare & Toggle Out   */
sbit at 0xC5	CMSR5	; /* P4.5        T2 Compare & Set/Reset Out*/
sbit at 0xC4	CMSR4	; /* P4.4        T2 Compare & Set/Reset Out*/
sbit at 0xC3	CMSR3	; /* P4.3        T2 Compare & Set/Reset Out*/
sbit at 0xC2	CMSR2	; /* P4.2        T2 Compare & Set/Reset Out*/
sbit at 0xC1	CMSR1	; /* P4.1        T2 Compare & Set/Reset Out*/
sbit at 0xC0	CMSR0	; /* P4.0        T2 Compare & Set/Reset Out*/

sbit at 0xDE	ENS1	;  /* S1CON.6     Enable Serial I/O         */
sbit at 0xDD	STA	;  /* S1CON.5     Start Flag                */
sbit at 0xDC	STO	;  /* S1CON.4     Stop Flag                 */
sbit at 0xDB	SI	;  /* S1CON.3     Serial I/O Interupt       */
sbit at 0xDA	AA	;  /* S1CON.2     Assert Acknowledge        */
sbit at 0xD9	CR1	;  /* S1CON.1     Clock Rate 1              */
sbit at 0xD8	CR0	;  /* S1CON.0     Clock Rate 0              */

sbit at 0xD7	CY	;  /* PSW.7       Carry Flag                */
sbit at 0xD6	AC	;  /* PSW.6       Aux Carry Flag            */
sbit at 0xD5	F0	;  /* PSW.5       Flag 0                    */
sbit at 0xD4	RS1	;  /* PSW.4       Register Bank Select 1    */
sbit at 0xD3	RS0	;  /* PSW.3       Register Bank Select 0    */
sbit at 0xD2	OV	;  /* PSW.2       Overflow Flag             */
sbit at 0xD1	F1	;  /* PSW.1       Flag 1                    */
sbit at 0xD0	P	;  /* PSW.0       Accumulator Parity Flag   */

sbit at 0xE8	ECT0	;  /* IEN1.0      Enable T2 Capture 0       */
sbit at 0xE9	ECT1	;  /* IEN1.1      Enable T2 Capture 1       */
sbit at 0xEA	ECT2	;  /* IEN1.2      Enable T2 Capture 2       */
sbit at 0xEB	ECT3	;  /* IEN1.3      Enable T2 Capture 3       */
sbit at 0xEC	ECM0	;  /* IEN1.4      Enable T2 Comparator 0    */
sbit at 0xED	ECM1	;  /* IEN1.5      Enable T2 Comparator 1    */
sbit at 0xEE	ECM2	;  /* IEN1.6      Enable T2 Comparator 2    */
sbit at 0xEF	ET2	;  /* IEN1.7      Enable T2 Overflow        */

sbit at 0xF8	PCT0	;  /* IP1.0       T2 Capture 0              */
sbit at 0xF9	PCT1	;  /* IP1.1       T2 Capture 1              */
sbit at 0xFA	PCT2	;  /* IP1.2       T2 Capture 2              */
sbit at 0xFB	PCT3	;  /* IP1.3       T2 Capture 3              */
sbit at 0xFC	PCM0	;  /* IP1.4       T2 Comparator 0           */
sbit at 0xFD	PCM1	;  /* IP1.5       T2 Comparator 1           */
sbit at 0xFE	PCM2	;  /* IP1.6       T2 Comparator 2           */
sbit at 0xFF	PT2	;  /* IP1.7       T2 Overflow               */

#endif